@@ -208,15 +208,34 @@ class ZipDataset(Dataset):
     def __init__(self, *datasets, **kwargs):
         """See `Dataset.zip()` for details."""
         super().__init__(datasets, **kwargs)
-        if not all(len(datasets[0]) == len(ds) for ds in datasets):
-            raise ValueError("lengths of input datasets are inconsistent")
         self.symbol = tuple([ds.symbol for ds in datasets])
 
     def __getitem__(self, index: int) -> Tuple:
-        return tuple(ds[index] for ds in self.items)
+        return tuple([ds[index%len(ds)] for ds in self.items])
+
+    def __getattr__(self, name):
+        if name in self.__dict__:
+            return self.__dict__[name]
+        if name.endswith('transform_funcs'):
+            if 'items' in self.__dict__:
+                items = self.__dict__['items']
+                ds=items[0]
+                if name in ds.__dict__:
+                    return ds.__dict__[name]
+        raise AttributeError("'{}' object has no attribute '{}'".format(type(self).__name__, name))
+
+    def __setattr__(self, name: str, value) -> None:
+        if name.endswith('transform_funcs'):
+            if 'items' in self.__dict__:
+                items = self.__dict__['items']
+                for ds in items:
+                    if hasattr(ds, name):
+                        setattr(ds,name,value)
+        else:
+            object.__setattr__(self, name, value)
 
     def __len__(self) -> int:
-        return len(self.items[0])
+        return len(self.items)
 
 
 class NumpyDataset(Dataset):
@@ -281,7 +300,9 @@ class ImageDataset(Dataset):
 
     def __getitem__(self, index: int):
         img = self.items[index]  # self.pop(index)
-        if isinstance(img, str):
+        if isinstance(img, str) and self.object_type==ObjectType.image_path:
+            return img
+        else:
             img = file2array(img)
 
         if not isinstance(img, np.ndarray):
@@ -312,6 +333,8 @@ class ImageDataset(Dataset):
         return img
 
     def data_transform(self, img_data):
+        if self.object_type==ObjectType.image_path:
+            return img_data
         if len(self.transform_funcs) == 0:
             return image_backend_adaption(img_data)
         if isinstance(img_data, np.ndarray):
@@ -359,6 +382,8 @@ class ImageDataset(Dataset):
             return img_data
 
 
+
+
 class MaskDataset(Dataset):
     def __init__(self, masks, class_names=None, object_type: ObjectType = ObjectType.label_mask,
                  get_image_mode: GetImageMode = GetImageMode.processed, symbol="mask", **kwargs):
@@ -1003,14 +1028,14 @@ class TextSequenceDataset(Dataset):
 
 
 class Iterator(object):
-    def __init__(self, data=None, label=None, mask=None, unpair=None, sample_filter=None, batch_size=8, mode='tuple', is_shuffe=True, buffer_size=None, workers=2, **kwargs):
+    def __init__(self, data=None, label=None, mask=None, unpair=None, sample_filter=None, batch_size=8, mode='tuple', is_shuffle=True, buffer_size=None, workers=2, **kwargs):
         self.is_paired_process = False
         self._data = None
         self._label = None
         self._unpair = None
         self.paired_process_symbols = []
 
-        self.is_shuffe = is_shuffe
+        self.is_shuffle = is_shuffle
 
 
 
@@ -1060,6 +1085,8 @@ class Iterator(object):
             ds = datasets[k]
             if isinstance(ds,TextSequenceDataset):
                 ds.element_spec = TensorSpec(shape=TensorShape([None,ds.sequence_length]), dtype=dtype.long,object_type=ds.object_type, name=ds.symbol)
+            elif ds.object_type==ObjectType.image_path:
+                ds.element_spec = TensorSpec(shape=TensorShape([None]),dtype=str,object_type=ds.object_type, name=ds.symbol)
             else:
                 if len(ds) > 0:
                     dataitem = ds[k]
@@ -1069,7 +1096,7 @@ class Iterator(object):
         self._batch_size = batch_size
         self.paired_transform_funcs = []
 
-        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffe, drop_last=False)
+        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffle, drop_last=False)
         self._sample_iter = iter(self.batch_sampler)
         if buffer_size is None:
             buffer_size = 8 * batch_size
@@ -1095,7 +1122,7 @@ class Iterator(object):
         else:
             self._label.is_paired_process = self._data.is_paired_process = self.is_paired_process = False
 
-        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffe, drop_last=False)
+        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffle, drop_last=False)
         self.batch_sampler.sample_filter = self.sample_filter
         self._sample_iter = iter(self.batch_sampler)
 
@@ -1107,12 +1134,12 @@ class Iterator(object):
     def label(self, value):
         self._label = value
 
-        if isinstance(self._label, (MaskDataset, ImageDataset, BboxDataset)) and isinstance(self._data, ImageDataset) and len(
-                self._label) == len(self._data):
+        if isinstance(self._label, (MaskDataset, ImageDataset, BboxDataset)) and isinstance(self._data, ImageDataset) and len( self._label) == len(self._data):
             self._label.is_paired_process = self._data.is_paired_process = self.is_paired_process = True
+
         else:
             self._label.is_paired_process = self._data.is_paired_process = self.is_paired_process = False
-        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffe, drop_last=False)
+        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffle, drop_last=False)
         self.batch_sampler.sample_filter = self.sample_filter
         self._sample_iter = iter(self.batch_sampler)
 
@@ -1125,7 +1152,7 @@ class Iterator(object):
         self._unpair = value
 
 
-        self.batch_sampler = BatchSampler(self, self._batch_size,is_shuffle=self.is_shuffe,  drop_last=False)
+        self.batch_sampler = BatchSampler(self, self._batch_size,is_shuffle=self.is_shuffle,  drop_last=False)
         self.batch_sampler.sample_filter = self.sample_filter
         self._sample_iter = iter(self.batch_sampler)
 
@@ -1143,7 +1170,7 @@ class Iterator(object):
     @batch_size.setter
     def batch_size(self, value):
         self._batch_size = value
-        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffe, drop_last=False)
+        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffle, drop_last=False)
         self.batch_sampler.sample_filter = self.sample_filter
         self._sample_iter = iter(self.batch_sampler)
         self.buffer_size = 8 * value
@@ -1163,7 +1190,7 @@ class Iterator(object):
                 spec = ds.element_spec
                 self.data_template[spec] = None
                 self._signature.outputs[ds.symbol] = spec
-            self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffe, drop_last=False)
+            self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffle, drop_last=False)
             self._sample_iter = iter(self.batch_sampler)
             return self._signature
 
@@ -1213,7 +1240,7 @@ class Iterator(object):
         for ds, result,value in zip(datasets, results.key_list,results.value_list):
             spec = TensorSpec.tensor_to_spec(value, object_type=ds.object_type,need_exclude_batch_axis=True,is_singleton=True, name=ds.symbol)
             self._signature.outputs[ds.symbol] = spec
-        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffe, drop_last=False)
+        self.batch_sampler = BatchSampler(self, self._batch_size, is_shuffle=self.is_shuffle, drop_last=False)
         self._sample_iter = iter(self.batch_sampler)
         return self._signature
 
@@ -1351,7 +1378,7 @@ class MetricIterator(Iterator):
 
         self._batch_size = batch_size
         self.paired_transform_funcs = []
-        self.batch_sampler = BatchSampler(self, self._batch_size,is_shuffle=self.is_shuffe, drop_last=False)
+        self.batch_sampler = BatchSampler(self, self._batch_size,is_shuffle=self.is_shuffle, drop_last=False)
         self._sample_iter = iter(self.batch_sampler)
         self.buffer_size = 10
         self.out_queue = Queue.Queue(maxsize=self.buffer_size)
