@@ -199,28 +199,37 @@ class _ClassificationLoss(Loss):
         else:
             pass
 
-        output_exp = exp(output)
 
-        if (ndim(output) >= 1 and 'float' in str(output.dtype) and reduce_min(output) >= 0 and reduce_max(output) <= 1):
+        if ndim(output) >= 1 and 'float' in str(output.dtype) and reduce_min(output) >= 0 :
             self.is_logsoftmax = False
-            self.from_logits = True
+            if reduce_max(output) > 1:
+                output=output/reduce_max(output)
+
+            sum_output=reduce_sum(output,axis=self.axis,keepdims=True)
+            if reduce_max(abs(sum_output-ones_like(sum_output)))<1e-3:
+                self.from_logits = True
+            else:
+                output=output/clip(sum_output,1e-7,1-1e-7)
+                self.from_logits = True
+
             if self.auto_balance and self.label_statistics is not None:
-                if int_shape(output)[1] == len(self.label_statistics):
+                if self.num_classes== len(self.label_statistics):
                     new_shp = [1] * len(int_shape(output))
                     new_shp[1] = len(self.label_statistics)
                     output = output * clip(to_tensor(np.reshape(self.label_statistics.copy(), tuple(new_shp))), min=1e-7)
 
-            output = clip(output, min=1e-7, max=1 - 1e-7)
+            output = clip(output, min=1e-7, max=1)
 
-        elif (ndim(output) >= 1 and 'float' in str(output.dtype) and output_exp.min() >= 0 and output_exp.max() <= 1):
+        elif (ndim(output) >= 1 and 'float' in str(output.dtype) and reduce_max(output) <=0):
             self.is_logsoftmax = True
             self.from_logits = True
+            output = clip(output,min=-7.0,max=-1e-8)
             if self.auto_balance and self.label_statistics is not None:
                 if int_shape(output)[1] == len(self.label_statistics):
                     new_shp = [1] * len(int_shape(output))
                     new_shp[1] = len(self.label_statistics)
                     output = output - to_tensor(np.reshape(np.log(clip(self.label_statistics.copy(), min=1e-7)), tuple(new_shp)))
-            output = clip(output, max=- 1e-7)
+
         else:
             self.is_logsoftmax = False
             self.from_logits = False
@@ -241,14 +250,13 @@ class _ClassificationLoss(Loss):
             self.need_target_onehot = True
         if target.dtype == str2dtype('long'):
             self.is_target_onehot = False
-        elif target.dtype != str2dtype('long') and (target.min() >= 0 and target.max() <= 1 and abs(output_exp.sum(-1).mean() - 1) < 1e-4):
-            target = clip(target, min=1e-7, max=1 - 1e-7)
+        elif target.dtype != str2dtype('long') and (target.min() >= 0 and target.max() <= 1 and abs(target.sum(self.axis)-1).mean() < 1e-4):
+            target = clip(target, min=1e-7, max=1 )
             self.is_target_onehot = True
 
         # need target onehot but currently not
         if target.dtype == torch.long and self.need_target_onehot == True and self.is_target_onehot == False:
             target = make_onehot(target, num_classes=self.num_classes, axis=self.axis).to(get_device())
-            target = clip(target, min=1e-7, max=1 - 1e-7)
             target.require_grads = False
             self.is_target_onehot = True
             if self.label_smooth:
@@ -281,7 +289,7 @@ class _ClassificationLoss(Loss):
     def _handel_abnormal(self, loss):
         if any_abnormal_number(loss):
             sys.stderr.write('{0} has abnormal value,trident automatically replace these abnormal value to zero.\n'.format(self.name))
-            loss = where(is_nan(loss), ones_like(loss, requires_grad=True) * 1e-7, loss)
+            loss = where(is_abnormal_number(loss), ones_like(loss, requires_grad=False) * 1e-7, loss)
         return loss
 
     def forward(self, output: Tensor, target: Tensor, **kwargs) -> 'loss':
@@ -603,14 +611,13 @@ class CrossEntropyLoss(_ClassificationLoss):
             else:
                 reshape_shape = [1] * ndim(output)
                 reshape_shape[self.axis] = self.num_classes
-                sample_weight = sample_weight.view(*reshape_shape)
                 sample_weight = self.sample_weight.view(*reshape_shape) * self.ignore_index_weight.view(*reshape_shape)
 
             if not self.is_logsoftmax:
                 return -reduce_sum(target * F.log_softmax(output, dim=self.axis) * sample_weight, axis=self.axis)
             else:
                 return -reduce_sum(target * output * sample_weight, axis=self.axis)
-        return loss
+
 
 
 class NLLLoss(_ClassificationLoss):
